// ParserIntervalles.h: interface for the ParserIntervalles class.
//
//////////////////////////////////////////////////////////////////////

#ifndef PARSER_INTERVALLES_H
#define PARSER_INTERVALLES_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <vector>
#include "Variable.cpp"
#include "Interval.h"
#include "iboolean.h"
#include "Type.h"

#define V_PI  3.1415926535897932384626433832795


/* actions */
typedef enum {
	S, /* shift */
	R, /* reduce */
	A, /* accept */
	E1, /* error: missing right parenthesis */
	E2, /* error: missing operator */
	E3, /* error: unbalanced right parenthesis */
	E4, /* error: invalid function argument */
	E5 /* error: typage */
} actEnum;



/* tokens */
typedef enum {
/* operators */
	tAdd, /* + */
	tSub, /* - */
	tMul, /* * */
	tDiv, /* / */
	tPow, /* ^ (power) */
	tUmi, /* - (unary minus) */
	tFact, /* f(x): factorial */
	tPerm, /* p(n,r): permutations */
	tComb, /* c(n,r): combinations */
	tAbs,	/* Abs(x): */
	tCos,	/* Cos(x): */
	tCotan,	/* Cotan(x): */
	tExp,	/* Exp(x): */
	tLog,	/* Log(x): */
	tMax,	/* Max(x,y): */
	tMin,	/* Min(x,y): */
	tPower,	/* Power(x,n): */
	tSin,	/* Sin(x): */
	tSqr,	/* Sqr(x): */
	tSqrt,	/* Sqrt(x): */
	tTan,	/* Tan(x): */
	tInter,	/* Interval(x,y): */
	tIn,	/* In(x,y):*/
	tInters,/* Inter(x,y):*/
	tNot,	/* not(x):*/
	tUnion, /* Union(x,y)*/
	tDisjoint,/* Disjoint(x,y)*/
	tEgale,	/* Egale(x,y)*/
	tEt,	/* x et y*/
	tOu,	/* x ou y*/
	tComa, /* comma */
	tLpr, /* ( */
	tRpr, /* ) */
	tEof, /* end of string */
	tMaxOp, /* maximum number of operators */
	/* non-operators */
	tVal /* value */
} tokEnum;


#define MAX_OPR 50
#define MAX_VAL 50


typedef char Separateur;
struct functionToken
{
    char *fname;
	tokEnum tok;
};



typedef Variable<Type> variables;


class ParserIntervalles
{
	char chaine[255];
	tokEnum tok; /* token */
	Type tokval; /* token value */
	char opr[MAX_OPR]; /* operator stack */
	Type val[MAX_VAL]; /* value stack */
	int oprTop, valTop; /* top of operator, value stack */
	bool firsttok; /* true if first token */
	void insertSpace(char * str);/*spare les tokens*/
	int error(char *msg);
	int gettok(void);
	int shift(void);
	int reduce(void);
	int parse(void);

	bool retour;
	vector<Separateur*> listeSeparateur;//liste des sprateus

	vector<variables*> listeVariable;//liste des variables

public:
		
	bool getValeur(char *str,Type& val);
	bool setValeur(char *str,Type& val);
	bool eval(char* str,Type& res);
	bool eval(char* str);

	void Init();
	int getNextToken(char*chaine,char*token);
	functionToken* getNameFunction(int num);
	


	ParserIntervalles();
	virtual ~ParserIntervalles();


	friend double	fact(double n);
	friend double	Abs(const double&);
	friend double	Cos(const double&);
	friend double	Cotan(const double&);
	friend double	Exp(const double&);
	friend double	Log(const double&);
	friend double	Max(const double&,const double&);
	friend double	Min(const double&,const double&);
	friend double	Power(const double&,double);
	friend double	Sin(const double&);
	friend double	Sqr(const double&);
	friend double	Sqrt(const double&);
	friend double	Tan(const double&);


	friend interval	fact(interval n);
	friend interval	Power(const interval& val,interval indice);
        friend iboolean	Non(const iboolean& val1);
        friend iboolean	Ou(const iboolean& val1,const iboolean& val2);

private:
	void doAffectation(char *variable,Type val);

	char* getVariableAffectation(char *str);
	int countOccurrence(const char*str,char* chaine);
	void splitExpression(char*src,char* left,char*right);
	bool isVariable(char *str);
};

#endif
