// Vecteur.cpp: implementation of the Vecteur class.
//
//////////////////////////////////////////////////////////////////////

#include "Vecteur.h"
#include "iostream.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
extern bool nonNul(double d);

Vecteur::Vecteur()
{
	this->dx = 0.0;
	this->dy = 0.0;
	this->dz = 0.0;
	this->dw = 1.0;
}

Vecteur::~Vecteur()
{

}

double Vecteur::get(int i)
{
	switch(i)
	{
		case 0:
			return dx;
		case 1:
			return dy;
		case 2:
			return dz;
		case 3:
			return dw;
	}
	
	return 0;
}

void Vecteur::set(int i,double d)
{
	switch(i)
	{
		case 0:
			dx = d;
		break;
		case 1:
			dy = d;
		break;
		case 2:
			dz = d;
		break;
		case 3:
			dw = d;
		break;
	}
}

Vecteur::Vecteur(const Vecteur &vect)
{
	this->dx = vect.dx;
	this->dy = vect.dy;
	this->dz = vect.dz;
	this->dw = 1.0;
}

Vecteur::Vecteur(double tab[3])
{	
	this->dx = tab[0];
	this->dy = tab[1];
	this->dz = tab[2];
	this->dw = 1.0;
}


void Vecteur::toFloat(float *tab)
{
	tab[0] = float(this->dx);
	tab[1] = float(this->dy);
	tab[2] = float(this->dz);
	tab[3] = float(this->dw);
}

bool Vecteur::operator <(Vecteur v)
{
	return true;
}

void Vecteur::operator+=(Vecteur v)
{
	this->dx += v.dx;
	this->dy += v.dy;
	this->dz += v.dz;
}

bool Vecteur::operator ==(Vecteur v)
{
	return (this->dx == v.dx	&&
			this->dy == v.dy	&&
			this->dz == v.dz);
}

void Vecteur::operator *=(float coeff)
{
	this->dx*=coeff;
	this->dy*=coeff;
	this->dz*=coeff;

	if(!nonNul(this->dx))
		this->dx = 0.0;

	if(!nonNul(this->dy))
		this->dy = 0.0;

	if(!nonNul(this->dz))
		this->dz = 0.0;
}

void Vecteur::operator /=(float coeff)
{
	this->dx/=coeff;
	this->dy/=coeff;
	this->dz/=coeff;

	if(!nonNul(this->dx))
		this->dx = 0.0;

	if(!nonNul(this->dy))
		this->dy = 0.0;

	if(!nonNul(this->dz))
		this->dz = 0.0;
}

Vecteur Vecteur::operator *(float coeff)
{
	Vecteur v;

	v = *this;
	
	v.dx*=coeff;
	v.dy*=coeff;
	v.dz*=coeff;

	if(!nonNul(v.dx))
		v.dx = 0.0;

	if(!nonNul(v.dy))
		v.dy = 0.0;

	if(!nonNul(v.dz))
		v.dz = 0.0;

	return v;
}


Vecteur::Vecteur(double dx, double dy, double dz)
{
	this->dx = dx;
	this->dy = dy;
	this->dz = dz;
	this->dw = 1.0;
	if(!nonNul(this->dx))
		this->dx = 0.0;

	if(!nonNul(this->dy))
		this->dy = 0.0;

	if(!nonNul(this->dz))
		this->dz = 0.0;
}

Vecteur Vecteur::prodVectoriel (Vecteur a , Vecteur b)
{	
	Vecteur res;
	
	res.dx = a.dy * b.dz - a.dz * b.dy;
	res.dy = a.dz * b.dx - a.dx * b.dz;
	res.dz = a.dx * b.dy - a.dy * b.dx;
	
	return res;
}

Vecteur Vecteur::prodVectoriel(Vecteur a)
{
	return Vecteur::prodVectoriel(*this,a);
}


double Vecteur::operator*(Vecteur vect)
{
	double prod;

	prod = this->dx*vect.dx + this->dy*vect.dy + this->dz*vect.dz;

	return prod;
}

double Vecteur::norm()
{
	return sqrt(dx*dx + dy*dy + dz*dz);
}

double Vecteur::angle(Vecteur v)
{
	return ( (*this * v) / (norm() * v.norm()) );
}

Vecteur::Vecteur(float* pt1, float* pt2)
{
	this->dx	=	pt2[0] - pt1[0];
	this->dy	=	pt2[1] - pt1[1];
	this->dz	=	pt2[2] - pt1[2];
}

void Vecteur::normaliser()
{
	double som;

	som = sqrt( dx*dx + dy*dy + dz*dz );

	if(som!=0)
	{
		dx/=som;
		dy/=som;
		dz/=som;
	}
}


double Vecteur::cosinus(Vecteur u, Vecteur v)
{
	u.normaliser();
	v.normaliser();
	
	return u*v;
}

double Vecteur::sinus(Vecteur u, Vecteur v, Vecteur w)
{
	u.normaliser();
	v.normaliser();
	w.normaliser();
	
	Vecteur pv;
	pv = Vecteur::prodVectoriel(u, v);
	
	return pv*w;
}

Vecteur Vecteur::projeteXY(void)
{
	Vecteur res;
	
	res.dx = dx;
	res.dy = dy;
	res.dz = 0;
	
	return res;
}

Vecteur Vecteur::projeteXZ(void)
{
	Vecteur res;
	res.dx = dx;
	res.dy = 0;
	res.dz = dz;
	
	return res;
}

Vecteur Vecteur::projeteYZ(void)
{
	Vecteur res;
	res.dx = 0;
	res.dy = dy;
	res.dz = dz;
	
	return res;
}

void Vecteur::affiche()
{
	cout << "[" << dx << ", " << dy << ", " << dz << "]" << endl;
}
