#ifndef _VECTOR_H
#define _VECTOR_H
#include <stdlib.h>
#include <string.h>

template<class type>
class Vector
{

private:

	typedef struct
	{
		type * pointeur;	//quoi qu'il arrive, on stoke que des pointeurs
	}stockpoint;

	inline void mem_Alloc(void);

	bool isFreeAll;

public:

	stockpoint * mzone;		//tableau stockant les donnes, et non plus une liste chain

	int nbrelement;			//nombre d'lments

	Vector(void);
	~Vector(void);

	inline void freeAll();										// Supprime les OBJETS contenus dans la Vector (liberation des ressources)
																// Supprime galement le Vector en lui meme

	inline void expand(const int& nbr);							// Ajoute n elements en fin de liste sans prciser le contenu
		
	inline void dynacopy(Vector<type> * liste);					// Ajoute les lments de liste en fin de notre liste
	inline void addElement(type * ceci);						// Ajoute ceci en fin de liste
	inline void addElementUnique(type * ceci);					// Ajoute ceci si ceci n'est pas dja dans la liste

	inline void addElement(const int& index,type * ceci);		// Place ceci  la nieme position

	inline bool find(type * ceci,int &index);					// True si ceci est prsent - Place son index dans &index - PREMIERE OCCURANCE TROUVEE
	inline bool find(type * ceci);								// True si ceci est prsent - PREMIERE OCCURANCE TROUVEE
	inline bool find(type * ceci, int debut, int fin);			// True si ceci est prsent entre l'indice dbut et l'indice fin


	inline void removeElement(type * ceci);						// Supprime le premier element contenant ceci (ne supprime pas l'objet, seulement son pointeur dans la liste...)
	inline type * removeElementAt(const int& index);				// Supprime le nieme element ( meme remarque )
	inline void limite(int nbrelement);							// Limite la liste  nbrelement ( le solde est supprim - meme remarque)

	inline type * get(const int& index);						// Retourne le nieme element
	inline int	size(void);										// Retourne le nombre d'elements

	inline void flush(void);									// Vide la liste - NE TOUCHE PAS AUX OBJETS
	inline void clean(void);									// Vide la liste et SUPPRIME LES OBJETS - Ne supprime PAS la liste elle mme

	inline void swap(int a,int b);								// Permute l'emplacement des elements d'indices a et b dans la liste
};


/* --------------------
    Membres definition
   -------------------- */


// Constructeur
template<class type>
Vector<type>::Vector(void)
{	
	isFreeAll = false;
	nbrelement = 0;
	mzone = NULL;
}

// Destructeur
template<class type>
Vector<type>::~Vector(void)
{
	free(mzone);
}

// Vide la liste ET SUPPRIME LES OBJETS - Ne supprime PAS la Vector elle mme
template<class type>
inline void Vector<type>::clean()
{
	register int i;
	for (i=0;i<nbrelement;i++)
		delete get(i);
	this->flush();
}

// Liberation des ressources - Supprime LES OBJETS points par la Vector AINSI que la Vector en elle meme
template<class type>
inline void Vector<type>::freeAll()
{
	register int i;
	isFreeAll=true;
	for (i=0;i<nbrelement;i++) delete get(i);
	delete this;
}

// Vide la liste de tous ses elements - NE SUPPRIME PAS LES OBJETS
template<class type>
inline void Vector<type>::flush(void)
{
	nbrelement = 0;
	free(mzone);
	mzone = NULL;
}

// Allocation memoire
template<class type>
inline void Vector<type>::mem_Alloc(void)
{
	mzone = (stockpoint*)realloc(mzone,nbrelement<<2);		// stockpoint est sens faire 4 octets = taille d'un pointeur

	// si mzone est NULL -> erreur d'allocation...
}

// Ajoute n elements en fin de liste sans prciser leur contenu
template<class type>
inline void Vector<type>::expand(const int& nbr)
{
	nbrelement+=nbr;
	mem_Alloc();
}

// Ajoute les lments de la Vector liste en fin de liste
template<class type>
inline void Vector<type>::dynacopy(Vector<type> * liste)
{
	if (liste->nbrelement==0)
		return;

	int last = nbrelement;
	expand(liste->nbrelement);
	memcpy(&mzone[last], liste->mzone, liste->nbrelement<<2);

}

// Ajoute ceci dans la liste
template<class type>
inline void Vector<type>::addElement(type * ceci)
{
	nbrelement++;
	mem_Alloc();
	mzone[nbrelement-1].pointeur = ceci;
}

// Ajoute ceci si ceci n'est pas deja dans la liste
template<class type>
inline void Vector<type>::addElementUnique(type * ceci)
{
	if (!find(ceci)) addElement(ceci);
}


// Place ceci  la nieme position dans la liste
template<class type>
inline void Vector<type>::addElement(const int& index,type * ceci)
{
	mzone[index].pointeur = ceci; 
}

// Retourne le nombre d'elements dans la liste
template<class type>
inline int Vector<type>::size(void)
{
	return nbrelement;
}

// Retourne le nieme element de la liste
template<class type>
inline type * Vector<type>::get(const int& index)
{
	if ( nbrelement > index ) 
		return mzone[index].pointeur;
	else
		return NULL;
}

// Retourne true si ceci est prsent dans la liste et place son index dans &index - PREMIERE OCCURANCE TROUVEE
template<class type>
inline bool Vector<type>::find(type * ceci,int &index)
{
	if (nbrelement==0)
		return false;

	register int count=0;
	register stockpoint * access;
	
	for (access=mzone;access<mzone+nbrelement;access++)
	{
		if (access->pointeur == ceci )
		{
			index = count;
			return true;
		}
		count++;
	}
	return false;
}

// Retourne true si ceci est prsent dans la liste
template<class type>
inline bool Vector<type>::find(type * ceci)
{
	if (nbrelement==0)
		return false;

	register stockpoint * access;
	
	for (access=mzone;access<mzone+nbrelement;access++)
		if (access->pointeur == ceci )
			return true;

	return false;
}

// Retourne true si ceci est prsent dans la liste entre l'indice debut et l'indice fin
template<class type>
inline bool Vector<type>::find(type * ceci, int debut, int fin)
{
	if (nbrelement==0)
		return false;

	register stockpoint * access;
	
	for (access=mzone+debut;access<mzone+fin;access++)
		if (access->pointeur == ceci )
			return true;

	return false;

}



// Supprime le premier element de la liste contenant ceci
template<class type>
inline void Vector<type>::removeElement(type * ceci)
{
	if (isFreeAll)	return;

	int index;
	if ( find(ceci,index) )
		removeElementAt(index);
}

// Supprime le nieme element de la liste (base0)
template<class type>
inline type* Vector<type>::removeElementAt(const int& index)
{
	
	type* ret = NULL;

	ret = get(index);

	if (isFreeAll)	return NULL;

	if (nbrelement<2)
	{
		nbrelement=0;
		free(mzone);
		mzone=NULL;
		return ret;
	}



	if (index<nbrelement-1)
		memmove(&mzone[index],&mzone[index+1], (nbrelement-(index+1))*4 );

	--nbrelement;

	mem_Alloc();

	return ret;
}

// Limite la liste  nbrelement ( le solde est supprim )

template<class type>
inline void Vector<type>::limite(int nbrelement)
{
	this->nbrelement=nbrelement;
	mem_Alloc();
}

// Permute l'emplacement des elements d'indices a et b dans la liste
template<class type>
inline void Vector<type>::swap(int a,int b)
{
	register type * tampon;

	tampon = (mzone+a)->pointeur;

	(mzone+a)->pointeur = (mzone+b)->pointeur;
	(mzone+b)->pointeur = tampon;
}


#endif
