#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "interval.h"



const double PI_2 = M_PI_2;
const double TROIS_PI_2 = 3*PI_2;
const double CINQ_PI_2 = 5*PI_2;
const double SEPT_PI_2 = 7*PI_2;
const double DEUX_PI = 4*PI_2;
const double PI = 2*PI_2;

double Arccossin(const double& x, const double& y)
{   if (y>0) return(acosl(x));
    else return(-acosl(x));
}
//----------------------------------------------------------------------
double Arg(const double& x, const double& y)
{   double r=sqrt(x*x+y*y);
    if (r==0) return 0;
    return Arccossin(x/r,y/r);
}
//----------------------------------------------------------------------
interval::interval ()
{   inf=0;  sup=-999;   // -999 <=> Not a Number
    isInterval = false;
    isEmpty=false;
}
//----------------------------------------------------------------------
interval::interval (double m)
{   inf=m;  sup=-999;   // -999 <=> Not a Number
    isInterval = false;
    isEmpty=false;
}
//----------------------------------------------------------------------
interval::interval (const double& a, const double& b)
{   if  (a>b)   {inf = b ; sup = a;}
    else        {inf = a ; sup = b;}
    isInterval = (a!=b);
    isEmpty=false;
}
//----------------------------------------------------------------------
//interval::interval (const interval& a) {*this = a;}
interval::interval (const interval& a )
{ inf  = a.inf;
  sup  = a.sup;
  isInterval=a.isInterval;
  if (inf==sup) isInterval=false;
  isEmpty=a.isEmpty;
}
//*********************************************************************
//***********************      OPERATEURS            ******************
//*********************************************************************
interval operator+ (const interval& x,const interval& y)
{   if (x.isEmpty) return x;
    if (y.isEmpty) return y;
    interval z(x.GetInf()+y.GetInf());
    if ((x.isInterval)||y.isInterval)
    {   z.isInterval=true;
        z.SetSup(x.GetSup()+y.GetSup());
    }
    return(z);
};
//----------------------------------------------------------------------
interval operator- (const interval& a)
{   if (a.isEmpty) return a;
    if  (a.isInterval)  return(interval(-(a.GetSup()),-(a.GetInf())));
    else                return(-a.GetInf());
}
//----------------------------------------------------------------------
interval operator- (const interval& x, const interval& y)
{   return(x+(-y));};
//----------------------------------------------------------------------
interval operator* (const interval& x, const interval& y)
{   interval r;
    if (x.isEmpty) return x;
    if (y.isEmpty) return y;
    double x1=x.GetInf(),y1=y.GetInf(),x2=x.GetSup(),y2=y.GetSup();
    if ((x.isInterval||y.isInterval)==false) return(interval(x1*y1));
    if (x1>=0)      {   if  (y1>=0) return interval(x1*y1,x2*y2);   else
                        if  (y2<=0) return interval(x2*y1,x1*y2);   else
                                    return interval(x2*y1,x2*y2);}
    else if (x2<=0) {   if  (y1>=0) return interval(x1*y2,x2*y1);   else
         if (y2<=0)                 return interval(x2*y2,x1*y1);   else
                                    return interval(x1*y2,x1*y1);}
    else {  if (y1>=0)              return interval(x1*y2,x2*y2);   else
            if (y2<=0)              return interval(x2*y1,x1*y1);   else
                                    return interval(min(x1*y2,x2*y1),max(x1*y1,x2*y2));}
}
//----------------------------------------------------------------------
interval operator/  (const interval& a, const interval& b)
{   interval r;
    if (a.isEmpty) return a;
    if (b.isEmpty) return b;
    if (!b.isInterval)
        {double b1=ToReel(b);
         if (b1==0) return(interval(-oo,oo));
         else return (1/b1)*a;
        }
    if (Inf(b) > 0)
    {   if      (Inf(a)>=0) return interval(Inf(a)/Sup(b),Sup(a)/Inf(b));
        else if (Sup(a)<0)  return interval(Inf(a)/Inf(b),Sup(a)/Sup(b));
        else                return interval(Inf(a)/Inf(b),Sup(a)/Inf(b));}
    else if (Sup(b)<0)
    {   if      (Inf(a)>=0) return interval(Sup(a)/Sup(b),Inf(a)/Inf(b));
        else if (Sup(a)<0)  return interval(Sup(a)/Inf(b),Inf(a)/Sup(b));
        else                return interval(Sup(a)/Sup(b),Inf(a)/Sup(b));}
    else                    return interval(-oo,oo);
}
//----------------------------------------------------------------------
ostream& operator<< (ostream& os, const interval& a)
{   if      (a.isEmpty)     os<<"EmptyInterval";
    else if (a.isInterval)  os << "[" << a.GetInf() << ", " << a.GetSup() << "] ";
    else                    os << "[" << a.GetInf() << "] ";
    return os;
}
//*********************************************************************
//**********************      FONCTION MEMBRES      *******************
//*********************************************************************

//----------------------------------------------------------------------
double interval::GetInf (void) const {return inf;};
//----------------------------------------------------------------------
double interval::GetSup (void) const
{   if (isInterval) {return sup;}
        else return inf;
}
//----------------------------------------------------------------------
void interval::SetInf(double a) {inf=a;}
//----------------------------------------------------------------------
void interval::SetSup(double a) {if (isInterval) sup=a; else ERROR;}
//----------------------------------------------------------------------

//*********************************************************************
//******************    FONCTION NON MEMBRES     **********************
//*********************************************************************

interval Abs  (const interval& a)
{   if (!a.isInterval) return fabs(Inf(a));
    interval r(fabs(Inf(a)),fabs(Sup(a)));
    if  (In(0,a)) {r.SetInf(0); r.isInterval=true;}
    return r;
}
//----------------------------------------------------------------------
double AbsMax (interval x)              // Absolute maximum of
{   double a = fabs(Inf(x)), b = fabs(Sup(x));
   // if (a > b) return a else  return b;
    return max(a,b);
}
//----------------------------------------------------------------------
double Center (const interval& a)
{   if (a.isInterval) return (Sup(a)+Inf(a))/2; return Inf(a); }
//----------------------------------------------------------------------
interval Cos(const interval& a) {return Sin(a+PI_2);}
//----------------------------------------------------------------------
bool Disjoint(const interval& a, const interval& b)
{   if (a.isEmpty ||b.isEmpty) return true;
    return (Sup(a)<Inf(b))||(Sup(b)<Inf(a));
}
//----------------------------------------------------------------------
void Egal (interval &y, interval& x)  //contract x,y w.r.t. the constraint y=x)
{   y=Inter(x,y); x=y; }
//----------------------------------------------------------------------
interval Exp(const interval& a)
{   double r1,r2;
    if (Inf(a)>20) r1=oo; else r1=exp(Inf(a)) ;
    if (a.isInterval)
    {   if (Sup(a)>20) r2=oo; else r2=exp(Sup(a));
        return interval(r1,r2);
    }
    return r1;
}
//----------------------------------------------------------------------
bool In(double a,const interval& b)
{   if (b.isEmpty) return false;
    return (Inf(b)<=a && a<=Sup(b));
}
//----------------------------------------------------------------------
iboolean In(const interval& F, const interval& Y)
{   if (Disjoint (F,Y)) return false;
    if (Subset   (F,Y)) return true;
    return iboolean(iperhaps);
}
//----------------------------------------------------------------------
double Eloignement(const interval& x, const interval& y)
{   if ((x.isEmpty)||(y.isEmpty)) return oo;
    double r=max(0.0,Inf(x)-Sup(y));
    r=max(r,Inf(y)-Sup(x));
    return r;
}
//----------------------------------------------------------------------
double Inf (const interval& a)    {return(a.GetInf());};
//----------------------------------------------------------------------
interval Inflate(const interval& a,double eps)
{   interval r(Inf(a)-eps,Sup(a)+eps);
    r.isEmpty=a.isEmpty;
    r.isInterval=true;
    return interval(r);
}
//----------------------------------------------------------------------
interval Inter(const interval &a, const interval &b)
{   interval r(1,2);
    r.isEmpty=(a.isEmpty || b.isEmpty);
    if ((Inf(a)>Sup(b))||(Inf(b)>Sup(a))) r.isEmpty=true;
    if (Inf(a)<Inf(b)) r.inf=Inf(b); else r.inf=Inf(a);
    if (Sup(a)>Sup(b)) r.sup=Sup(b); else r.sup=Sup(a);
    return interval(r);
}
//----------------------------------------------------------------------
void Min1(double& zmin0,double& zmin1,double a,double b,double c)
{   if ((a<=b)&&(a<=c)) {zmin0=a; zmin1=min(b,c); return;};
    if ((b<=a)&&(b<=c)) {zmin0=b; zmin1=min(a,c); return;};
    if ((c<=b)&&(c<=a)) {zmin0=c; zmin1=min(a,b); return;};
    return;
}
//----------------------------------------------------------------------
void Max1(double& zmax0,double& zmax1,double a,double b,double c)
{   if ((a>=b)&&(a>=c)) {zmax0=a; zmax1=max(b,c); return;};
    if ((b>=a)&&(b>=c)) {zmax0=b; zmax1=max(a,c); return;};
    if ((c>=b)&&(c>=a)) {zmax0=c; zmax1=max(a,b); return;};
}
//----------------------------------------------------------------------
void Inter1(interval& r0, interval& r1, const interval &a, const interval &b, const interval &c)
{   interval r(-999);
    if (a.isEmpty) {r0=a;r1=Inter(b,c);return;};
    if (b.isEmpty) {r0=b;r1=Inter(a,c);return;};
    if (c.isEmpty) {r0=c;r1=Inter(a,b);return;};
    double zmin0,zmax0,zmin1,zmax1;
    //Min1(zmin0,zmin1,Inf(a),Inf(b),Inf(c));
    //Max1(zmax0,zmax1,Sup(a),Sup(b),Sup(c));
    Max1(zmin0,zmin1,Inf(a),Inf(b),Inf(c));
    Min1(zmax0,zmax1,Sup(a),Sup(b),Sup(c));
    if (zmin0>zmax0)  r0.isEmpty=true; else r0=interval(zmin1,zmax1);
    if (zmin1>zmax1)  r1.isEmpty=true; else r1=interval(zmin1,zmax1);
    return;
}
//----------------------------------------------------------------------
interval Union(const interval& a, const interval& b)
{interval r(1,2);
 if (a.isEmpty) return b;
 if (b.isEmpty) return a;
 r.isEmpty=false;
 if (Inf(a)<Inf(b)) r.inf=Inf(a); else r.inf=Inf(b);
 if (Sup(a)>Sup(b)) r.sup=Sup(a); else r.sup=Sup(b);
 return interval(r);}
//----------------------------------------------------------------------
void Sucre(interval& P, const interval& S)
{   if (Disjoint(P,S)||Subset(S,P)||Subset(P,S)) return;
    if (In(Inf(S),P)) {P.sup=Inf(S); return;}
    if (In(Sup(S),P)) {P.inf=Sup(S); return;}
}
//----------------------------------------------------------------------
interval Log(const interval& a)
{interval b=Abs(a);
 if (In(0,a)) return interval(-oo,log(Sup(a)));
 if (Sup(a)<0) return interval(-oo,-oo);
 return interval(log(Inf(b)),log(Sup(b)));}
//----------------------------------------------------------------------
interval Max (const interval& x,const interval& y)
{   return -Min(-x,-y); }
//----------------------------------------------------------------------
interval Min (const interval& x,const interval& y)
{   interval z;
    if (x.isEmpty || y.isEmpty) {z.isEmpty=true; return z;};
    double a = min(Inf(x),Inf(y));
    double b = min(Sup(x),Sup(y));
    if (a==b) return interval(a);
    return interval(a,b);
}
//----------------------------------------------------------------------
void Min (interval &z, interval& x, interval& y)  //contract x,y,z, w.r.t. the constraint z=min(x,y)
{   z=Inter(z,Min(x,y));
    if (z.isEmpty) {x.isEmpty=true; y.isEmpty=true; return;}
    x=Inter(x,interval(Inf(z),oo));
    y=Inter(y,interval(Inf(z),oo));
}
//----------------------------------------------------------------------
void Min (interval &y, interval& x1,interval& x2,interval& x3)  //contract y,x1,x2,x3, w.r.t. the constraint y=min(x1,x2,x3);
{   interval a=interval(-oo,oo);
    Min(a,x2,x3);
    Min(y,x1,a);
    Min(a,x2,x3);
}
//----------------------------------------------------------------------
interval Modulo(const interval& a, double x)
{   if ((Inf(a)>=0)&&(Inf(a)<x)) return (a);
    int k = (long)floorl((Inf(a)/x));
    double offset = x * k;
    return interval(Inf(a)-offset,Sup(a)-offset);
}
//----------------------------------------------------------------------
bool OverLap(const interval& a, const interval& b)
{   return (Sup(a)>Inf(b))&&(Sup(b)>Inf(a));}
//----------------------------------------------------------------------
interval Parabole(const interval& x,double a,double b,double c)
{   return (  a*Sqr(x+(b/(2*a)))-(b*b)/(4*a)+c);}
//----------------------------------------------------------------------
double Pioche (const interval& a) // Chose a random number inside a
{   return (a.GetInf()+(rand()*Width(a))/RAND_MAX);};
//----------------------------------------------------------------------
void Plus (interval &z, interval& x, interval& y)  //contract x,y,z, w.r.t. the constraint z=x+y)
{   z=Inter(z,x+y);
    if (z.isEmpty) {x.isEmpty=true; y.isEmpty=true; return;}
    x=Inter(x,z-y);
    y=Inter(y,z-x);
}
//----------------------------------------------------------------------
interval Sech (const interval& x)
{   double x1=Inf(x),x2=Sup(x);
    double y1=2.0/(exp(x1)+exp(-x1));
    double y2=2.0/(exp(x2)+exp(-x2));
    if (!x.isInterval)  return y1;
    if ((x1>=0)||(x2<=0))   return interval(y1,y2);
    if (fabs(x1)>fabs(x2))  return interval(y1,1);
    else                    return interval(y2,1);
}
//----------------------------------------------------------------------
interval Sin (const interval& a)
{   if (!a.isInterval) return sin(Inf(a));
    interval b;
    double sin1,sin2,r1,r2;
    b = Modulo(a,DEUX_PI);
    if (Width(a)>DEUX_PI) return (interval(-1,1));
    sin1=sin(Inf(b));   sin2=sin(Sup(b));
    if ((Inf(b) < TROIS_PI_2)&&(Sup(b) > TROIS_PI_2)) r1=-1.0;
    else if ((Inf(b) < SEPT_PI_2)&&(Sup(b) > SEPT_PI_2)) r1=-1.0;
    else r1=((sin1 < sin2)? sin1 : sin2);
    if ((Inf(b) < PI_2)&&(Sup(b) > PI_2)) r2=1.0;
    else if ((Inf(b) < CINQ_PI_2)&&(Sup(b) > CINQ_PI_2)) r2=1.0;
    else r2=((sin1 > sin2)? sin1 : sin2);
    return interval(r1,r2);
}
//----------------------------------------------------------------------
interval Sqr(const interval& a)
{   double a1=Inf(a),a2=Sup(a);
    if (!a.isInterval)  return a1*a1;
    if ((a1>=0)||(a2<=0))   return interval(a1*a1,a2*a2);
    if (fabs(a1)>fabs(a2))  return interval(0,a1*a1);
    else                    return interval(0,a2*a2);
}
//----------------------------------------------------------------------
interval Sqrt(const interval& a)   //Set square root (e.g. Sqrt([4,9])=[-3,3])
{   double a1=Inf(a),a2=Sup(a);
    interval r;
    if (a2>=0)  r=interval(-sqrt(a2),sqrt(a2));
    else        r.isEmpty=true;
    return r;
}
//----------------------------------------------------------------------
interval SqrtPos(const interval& a)  //positive square root (e.g. Sqrt([4,9])=[2,3])
{   double a1=Inf(a),a2=Sup(a);
    interval r;
    if (a2<0)  {r.isEmpty=true;  return r;};
    if (a1<0)  {a1=0;};
    r=interval(sqrt(a1),sqrt(a2));
    return r;
}
//----------------------------------------------------------------------
bool Subset(const interval& a,const interval& b)
{   return (Inf(b)<=Inf(a))&&(Sup(a)<=Sup(b));}
//----------------------------------------------------------------------
bool Subset(const interval& a, const interval& b,double epsilon)
{   return (Subset(a,b)&&((Inf(b)<Inf(a)-epsilon)||(Sup(a)<Sup(b)-epsilon)));}
//----------------------------------------------------------------------
double Sup (const interval& a)    {return a.GetSup();};
//----------------------------------------------------------------------------
static double Power (double x, int n, int RndMode )
{ int  ChangeRndMode;     // for x < 0 and odd n
  double p, z;
  ChangeRndMode = ( (x < 0.0) && (n % 2 == 1) );
  if (ChangeRndMode) {z=-x; RndMode=-RndMode;}  else     z= x;
  p = 1.0;
  switch (RndMode) {                             // Separate while-loops used
        case -1 :   while (n > 0)
                    {if (n % 2 == 1) p = p*z;   //--------------------------
                     n = n / 2; if (n > 0) z = z*z;}break;
        case +1 :   while (n > 0)
                    {if (n % 2 == 1) p = p*z;
                     n = n / 2;
                     if (n > 0) z = z*z;}break;  }
  if (ChangeRndMode)   return -p; else  return p;
}
//----------------------------------------------------------------------------
interval Power (const interval& x, int n)
{ int  m;   double Lower, Upper;
  if (n == 0) return(interval(1.0,1.0));
  if (n > 0)  m = n;  else  m = -n;
  if ( (0.0 < Inf(x)) || (m % 2 == 1) )
  {  Lower = Power(Inf(x),m,-1);    Upper = Power(Sup(x),m,+1);}
  else if (0.0 > Sup(x))
  {  Lower = Power(Sup(x),m,-1);    Upper = Power(Inf(x),m,+1);}
  else  {Lower = 0.0;Upper = Power(AbsMax(x),m,+1); }
  if (n > 0) return(interval(Lower,Upper));
  else   return(1.0/interval(Lower,Upper));    // if 0 in 'x'.
}
//----------------------------------------------------------------------
interval Tan (const interval& a)
{   if (!a.isInterval) return tan(Inf(a));
    if (Width(a)>PI) return (interval(-oo,oo));
    interval b = Modulo(a,PI);
    if (PI_2<=Inf(b)) {b=b-PI;}
    if (In(PI_2,b)) return (interval(-oo,oo));
    return interval(tan(Inf(b)),tan(Sup(b)));
}
//----------------------------------------------------------------------
interval Cotan (const interval& a)
{   if (!a.isInterval) return 1/tan(Inf(a));
    return Tan(PI_2-a);
}
//----------------------------------------------------------------------
double ToReel (const interval& a)
    {   if (a.isInterval) return -9999;
        return Inf(a);
    }
//----------------------------------------------------------------------
double Width (const interval& a)
    {   return (Sup(a)-Inf(a)); }
//----------------------------------------------------------------------
