//---------------------------------------------------------------------------
#include <stdlib.h>
#include "repere_xfig.h"
//---------------------------------------------------------------------------

void repere::ResetDivision()
{
  nb_division = 0;
}

void repere::IncDivision()
{
  nb_division ++;
}

int repere::GetNbdivision()
{
  return nb_division;
}

void repere::CreateFicXfig()
{
  if ( (f = fopen(nom_fic,"w")) == NULL)
    {
      cout << "Error !!! repere::CreateFicXfig() File " << nom_fic << " cannot be opened (write mode)" << endl;
      exit(1);
    }
  fprintf(f, XFIG_HEADER);
  fclose(f);
}

void repere::InitCoeff_x_y()
{
  coeff_x_a = ( xfig_x.GetSup() - xfig_x.GetInf()) / ( interval_x.GetSup() - interval_x.GetInf() );
  coeff_x_b = xfig_x.GetSup() - coeff_x_a * interval_x.GetSup();

  coeff_y_a = ( xfig_y.GetSup() - xfig_y.GetInf()) / ( interval_y.GetSup() - interval_y.GetInf() );
  coeff_y_b = xfig_y.GetSup() - coeff_y_a * interval_y.GetSup();
}

repere::repere (const char *nom)
{
  xfig_x = interval(XFIG_COOR_XMIN, XFIG_COOR_XMAX);
  xfig_y = interval(XFIG_COOR_YMIN, XFIG_COOR_YMAX);
  interval_x = interval(XMIN_DEF,XMAX_DEF);
  interval_y = interval(YMIN_DEF,YMAX_DEF);
  strcpy(nom_fic, nom);
  CreateFicXfig();
  InitCoeff_x_y();
  ResetDivision();
};

repere::repere ()
{
  
};



//---------------------------------------------------------------------------
repere::repere (const char *nom,double xmin,double xmax,double ymin,double ymax)
{
  xfig_x = interval(XFIG_COOR_XMIN, XFIG_COOR_XMAX);
  xfig_y = interval(XFIG_COOR_YMIN, XFIG_COOR_YMAX);
  interval_x = interval(xmin,xmax);
  interval_y = interval(ymin,ymax);
  strcpy(nom_fic, nom);
  CreateFicXfig();
  InitCoeff_x_y();
  ResetDivision();
 
};

//---------------------------------------------------------------------------
void repere::DrawBox (box& P, int color,int X,int Y)
{
  int box_min_x, box_max_x, box_min_y, box_max_y, grid;
  if (P.isEmpty) return;
  if ( (f = fopen(nom_fic, "a")) == NULL)
    {
      cout << "Error !!! repere::DrawBox() File " << nom_fic << " cannot be opened (append mode)" << endl;
      exit(1);
    }
  if ( (color != XFIG_BLUE) & (color != XFIG_RED) & (color != XFIG_YELLOW) )
    {
      cout << "Error !!! repere::DrawBox() color = " << color << " is not a valid color" << endl;
      exit(1);
    }

  box_min_x = (int) ( P[X].GetInf() * coeff_x_a + coeff_x_b);
  box_max_x = (int) ( P[X].GetSup() * coeff_x_a + coeff_x_b);
  
  if(box_min_x == box_max_x)
  {
  	box_min_x--;
  }
  
  box_min_y = (int) ( P[Y].GetInf() * coeff_y_a + coeff_y_b);
  box_max_y = (int) ( P[Y].GetSup() * coeff_y_a + coeff_y_b);
  
  if(box_min_y == box_max_y)
  {
  	box_min_y--;
  }

  if ( (box_max_x - box_min_x < 10 ) | (box_max_y - box_min_y < 10) )
    grid = 0;
  else
    grid = GRID;

  fprintf(f, XFIG_BOX_HEADER, grid, color);

  fprintf(f, "%d %d %d %d %d %d %d %d %d %d\n", box_min_x, box_min_y, box_max_x, box_min_y, box_max_x, box_max_y, box_min_x, box_max_y, box_min_x, box_min_y);
  fclose(f);
}

//---------------------------------------------------------------------------
void repere::DrawContour (box& P,int color)
{
  int box_min_x, box_max_x, box_min_y, box_max_y;
  if (P.isEmpty) return;
  if ( (f = fopen(nom_fic, "a")) == NULL)
  {
      cout << "Error !!! repere::WriteBox() File " << nom_fic << " cannot be opened (append mode)" << endl;
      exit(1);
  }
 
  box_min_x = (int) ( P[1].GetInf() * coeff_x_a + coeff_x_b);
  box_max_x = (int) ( P[1].GetSup() * coeff_x_a + coeff_x_b);
  box_min_y = (int) ( P[2].GetInf() * coeff_y_a + coeff_y_b);
  box_max_y = (int) ( P[2].GetSup() * coeff_y_a + coeff_y_b);
  

  /*On ecrit le rectangle*/
  fprintf(f, XFIG_CONTOUR_HEADER, color);
  fprintf(f, "%d %d %d %d %d %d %d %d %d %d\n", box_min_x, box_min_y, box_max_x, box_min_y, box_max_x, box_max_y, box_min_x, box_max_y, box_min_x, box_min_y);
    
  fclose(f);
}


//---------------------------------------------------------------------------
void repere::DrawLine(interval *P, int Taille)
{
  int compteur,line_min_y,line_max_y,line_x;
  
   if ( (f = fopen(nom_fic, "a")) == NULL)
    {
      cout << "Error !!! repere::DrawLineY() File " << nom_fic << " cannot be opened (append mode)" << endl;
      exit(1);
    }
  
  
  /*On joint les min du tableau*/
  fprintf(f, XFIG_LINE_HEADER,2,1, XFIG_BLUE,Taille);
  for(compteur = 0; compteur < Taille; compteur++)
  {
  	line_min_y = (int) ( P[compteur].GetInf() * coeff_x_a + coeff_x_b);
    line_x = (int) ( compteur * coeff_y_a + coeff_y_b);
	fprintf(f, "%d %d ", line_x,line_min_y);
  }
  fprintf(f,"\n");
  
 
  /*on joint les max*/
  fprintf(f, XFIG_LINE_HEADER,2,1, XFIG_RED,Taille);
  for(compteur = 0; compteur < Taille; compteur++)
  {
  	line_max_y = (int) ( P[compteur].GetSup() * coeff_x_a + coeff_x_b);
    line_x = (int) ( compteur * coeff_y_a + coeff_y_b);
	fprintf(f, "%d %d ", line_x,line_max_y);
  }
  fprintf(f,"\n");
  
  fclose(f);
}

//---------------------------------------------------------------------------
void repere::DrawBaton(interval *P, int Taille)
{
  int compteur,line_min_y,line_max_y,line_x;
  
   if ( (f = fopen(nom_fic, "a")) == NULL)
    {
      cout << "Error !!! repere::DrawBaton() File " << nom_fic << " cannot be opened (append mode)" << endl;
      exit(1);
    }
  
  
  /*On joint les deux par deux de maniere  former  du tableau*/
  for(compteur = 0; compteur < Taille; compteur++)
  {
    fprintf(f, XFIG_LINE_HEADER, 1,1, XFIG_BLUE,2);
	line_min_y = (int) ( P[compteur].GetInf() * coeff_y_a + coeff_y_b);
	line_max_y = (int) ( P[compteur].GetSup() * coeff_y_a + coeff_y_b);
	if (line_min_y == line_max_y)
	{
		line_min_y--;
	}

    line_x = (int) ( compteur * coeff_x_a + coeff_x_b);
	fprintf(f, "%d %d %d %d", line_x,line_min_y,line_x,line_max_y);
	fprintf(f,"\n");

  }
   
  fclose(f);
}

